package sockprog.example1_hello;

import java.io.*;
import java.net.Socket;

public final class MessageClient {
    public static void main(String[] args) { // [1]
        if(args.length < 3) {
            System.err.println("parameter not enough");
            System.exit(-1);
        }

        String host = args[0];
        int port = Integer.parseInt(args[1]);
        String message = args[2];
        
        MessageClient client = null;
        try {
            client = new MessageClient(host, port); // [2]
            String reply = client.send(message);    // [3]
            System.out.println(reply);
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            try {
                if(client != null) {
                    client.shutDown();  // [4]
                }
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }
    
    private final Socket socket;
    private final Writer writer;
    private final BufferedReader reader;    

    public MessageClient(String host, int port) // [5] 
    throws IOException {
        socket = new Socket(host, port);

        // writerの準備
        OutputStream out = socket.getOutputStream();
        OutputStreamWriter outWriter = 
            new OutputStreamWriter(out, ProtocolConstants.ENCODING);
        writer = new BufferedWriter(outWriter);

        // readerの準備
        InputStream in = socket.getInputStream();
        InputStreamReader inReader =
            new InputStreamReader(in, ProtocolConstants.ENCODING);
        reader = new BufferedReader(inReader);
    }
    public void shutDown() throws IOException { // [7]
        socket.close();
    }
    
    public String send(String message) // [8] 
    throws IOException {
        // 文字列の送信
        writer.write(message); // [9]
        writer.write(ProtocolConstants.LINE_SEPARATOR);
        writer.flush(); // [10]

        // 文字列の受信
        return reader.readLine();  // [11]
    }
    
}
