package sockprog.example2_reuse;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.Writer;
import java.net.Socket;


public final class MessageClient {
    public static void main(String[] args) {
        if(args.length < 3) {
            System.err.println("parameter not enough");
            System.exit(-1);
        }

        String host = args[0];
        int port = Integer.parseInt(args[1]);
        String message = args[2];
        
        MessageClient client = null;
        try {
            client = new MessageClient(host, port);
            String reply = client.send(message);
            System.out.println(reply);
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            try {
                if(client != null) {
                    client.shutDown();
                }
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }
    
    private final Socket socket;
    private final Writer writer;
    private final BufferedReader reader;    

    public MessageClient(String host, int port)
    throws IOException {
        socket = new Socket(host, port);

        // writerの準備
        OutputStream out = socket.getOutputStream();
        OutputStreamWriter outWriter = 
            new OutputStreamWriter(out, ProtocolConstants.ENCODING);
        writer = new BufferedWriter(outWriter);

        // readerの準備
        InputStream in = socket.getInputStream();
        InputStreamReader inReader =
            new InputStreamReader(in, ProtocolConstants.ENCODING);
        reader = new BufferedReader(inReader);
    }
    public void shutDown() throws IOException {
        socket.close();
    }
    
    public String send(String message) throws IOException {
        // 文字列の送信
        writer.write(message);
        writer.write(ProtocolConstants.LINE_SEPARATOR);
        writer.flush();

        // 文字列の受信
        return reader.readLine();
    }
    
}
