package sockprog.example3_shutdown;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.Writer;
import java.net.Socket;

public final class MessageClient {
    public static void main(String[] args) {
        if(args.length < 3) {
            System.err.println("parameter not enough");
            System.exit(-1);
        }

        String host = args[0];
        int port = Integer.parseInt(args[1]);
        String message = args[2];

        try {
            MessageClient client = new MessageClient(host, port);
            String reply = client.send(message);
            System.out.println(reply);
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    private final String host;
    private final int port;

    public MessageClient(String host, int port)
    throws IOException {
        this.host = host;
        this.port = port;
    }

    public String send(String message) throws IOException {
        Socket socket = null;

        try {
            socket = new Socket(host, port);  // [1]
            // writerの準備
            OutputStream out = socket.getOutputStream();
            OutputStreamWriter outWriter =
                new OutputStreamWriter(out, ProtocolConstants.ENCODING);
            Writer writer = new BufferedWriter(outWriter);

            // 文字列の書き込み
            writer.write(message);
            writer.flush();
            
            socket.shutdownOutput();  // [2]    

            // readerの準備
            InputStream in = socket.getInputStream();
            InputStreamReader inReader =
                new InputStreamReader(in, ProtocolConstants.ENCODING);
            BufferedReader reader = new BufferedReader(inReader);

            // 文字列の読み込み
            StringBuilder stringBuilder = new StringBuilder(); 
            for(int c = reader.read();
            c != -1; c = reader.read()) { // [3]
                stringBuilder.append((char)c);
            }
            return stringBuilder.toString();
        } finally {
            if(socket != null) {
                socket.close();
            }
        }
    }

}
