// -*- coding: utf-8 -*-
// ↑Emacsで文字コードを強制するときのタグです。

// 基本的にセミコロンは不要です。

package javareading.multicast

import javareading.multicast.MCConfig._ // '*'の代わりに'_'を使います。

import scala.collection.mutable._

// Javaのクラスもそのまま使用可能です。
import java.text.SimpleDateFormat
import java.util.Date

// ・値を持つだけのクラスならこの定義のみになります。
// ・通常の変数はvar、final相当の変数はvalで定義します。
// ・変数の型は変数名の後に書きます。
class Message(val cnt: Int, val name: String, val srcAddr: String, val srcPort: Int, val regTime: Long)

class Display extends Runnable {
   // ジェネリクスには'<'…'>'ではなく'['…']'を使います。
  val map = new HashMap[String, Message]

  def register(msgStr: String, srcAddr: String, srcPort: Int) {
    // 通常、var, valでの型指定は型推論により省略可能です。
    val pair = msgStr.trim.split("\\s+")
    val cnt = Integer.parseInt(pair(0)) // 配列の添字は'['…']'の中ではなく'('…')'の中に書きます。
    val name = pair(1)
    // コンストラクタやメソッドに引数が無ければカッコを省略できます。
    val msg = new Message(cnt, name, srcAddr, srcPort, System.currentTimeMillis)
    // Javaのsynchronize相当はメソッド形式で表記します。Scalaでは式と文の区別がないので、処理はカッコの中に全部書けます。
    map.synchronized(map += ((name, msg))) // コレクションの操作は演算子オーバーライドで定義されています。
  }

  // 表示処理です。
  def run {
    val fmt = new SimpleDateFormat("[yyyy-MM-dd HH:mm:ss]")
    while (true) {
      val now = System.currentTimeMillis
      val expireTime = now - DISP_EXPIRES
      // print系のメソッドはプレフィックス無しで記述可能です。メソッド自体はscala.Console objectに定義されています。
      println("Scala " + fmt.format(new Date(now)))
      map.synchronized(
        // コレクション全体の操作には、Rubyのようなメソッドチェーンが使えます。
        map.values.toList.sort { (a, b) => a.name < b.name }.foreach { (msg) =>
          if (msg.regTime < expireTime)
            map -= (msg.name)
          else
            printf("  %04d %s (%s/%d)\n", msg.cnt, msg.name, msg.srcAddr, msg.srcPort)
        }
      )
      println
      Thread.sleep(DISP_INTERVAL)
    }
  }
}
