// -*- coding: utf-8 -*-
// ↑Emacsで文字コードを強制するときのタグです。

// 基本的にセミコロンは不要です。

package javareading.multicast

import javareading.multicast.MCConfig._ // '*'の代わりに'_'を使います。

// Javaのクラスもそのまま使用可能です。
import java.io.IOException
import java.net._

object Receiver {
  // ・mainメソッドはobjectのメソッドとして定義します。
  // 配列も通常のクラスで扱います。
  // ジェネリクスには'<'…'>'ではなく'['…']'を使います。
  def main(args: Array[String]) {
    try {
      // 通常の変数はvar、final相当の変数はvalで定義します。
      val addr = InetAddress.getByName(
        // ・式と文の区別はないので、カッコの中に直接if文などが書けます。
	// ・配列の添字は'['…']'の中ではなく'('…')'の中に書きます。
        if (args.length >= 1 && args(0) == "6") {
          println("Mode: IPv6\n")
          MULTICAST_ADDR6 // ブロックの中の最後の式がif文の値となります。
        } else {
          println("Mode: IPv4\n")
          MULTICAST_ADDR4
        }
      )
      // 以下はJavaのコードをほぼ機械的に置き換えたに近いのですが、型推論によりすっきり書けます。
      val socket = new MulticastSocket(MULTICAST_PORT)
      socket.joinGroup(addr)
      val buffer = new Array[Byte](RECV_BUF_SIZE)
      val packet = new DatagramPacket(buffer, buffer.length)
      val disp = new Display
      new Thread(disp).start // 表示処理を別スレッドで起動します。
      while (true) {
        socket.receive(packet)
        val off = packet.getOffset
        val len = packet.getLength
        val srcAddr = packet.getAddress.getHostAddress
        val srcPort = packet.getPort
        disp.register(new String(buffer, off, len, "UTF-8"), srcAddr, srcPort)
      }
    } catch {
      // 例外のハンドリングはパターンマッチで表現します。
      case e:IOException => (e.printStackTrace)
    }
  }
}
