// -*- coding: utf-8 -*-
// ↑Emacsで文字コードを強制するときのタグです。

// 基本的にセミコロンは不要です。

package javareading.multicast

import javareading.multicast.MCConfig._ // '*'の代わりに'_'を使います。

// Javaのクラスもそのまま使用可能です。
import java.net._

object Sender {
  // ・mainメソッドはobjectのメソッドとして定義します。
  // ・変数の型は後に書きます。
  // ・ジェネリクスには'<'…'>'ではなく'['…']'を使います。
  def main(args: Array[String]) = {
    // var の代わりに val を使うと、final 相当(単一代入)になります。
    val addr = InetAddress.getByName( // Java のクラスがそのまま使えます。
      // ・式と文の区別はないので、カッコの中に直接if文などが書けます。
      // ・配列の添字は'['…']'の中ではなく'('…')'の中に書きます。
      if (args.length >= 1 && args(0) == "6") {
        println("Mode: IPv6") // System.out は不要です。
        MULTICAST_ADDR6 // ブロックの中の最後の式がif文の値となります。
      } else {
        println("Mode: IPv4")
        MULTICAST_ADDR4
      }
    )
    val socket = new DatagramSocket // コンストラクタやメソッドに引数が無ければカッコを省略できます。
    var cnt = 0
    while (true) {
      cnt += 1 // Scalaには++がありません。
      val data = (cnt + " " + SEND_NAME).getBytes("UTF-8")
      val packet = new DatagramPacket(data, data.length, addr, MULTICAST_PORT)
      socket.send(packet)
      Thread.sleep(SEND_INTERVAL)
    }
  }
}
